##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Hdlist.pm,v 1.30 2004/12/27 14:02:56 othauvin Exp $

package Hdlist;

use strict;
use warnings;

use DynaLoader;
use Exporter;

use Hdlist::Header;

our $VERSION = '0.01';
our @ISA = qw(DynaLoader Exporter);
our @EXPORT = qw(moduleinfo
                 readconfig querytag tagName tagValue expand rpmvercmp
                 stream2header rpm2header
                 installsrpm
                 setverbosity setlogcallback format_rpmpb
                 rpmresign 
                 newdb parserpms);
our %EXPORT_TAGS = (
    rpmlib => [qw(getosname getarchname dumprc dumpmacros rpmvercmp setverbosity setlogcallback
                  rpmlog)],
    macros => [qw(add_macros del_macros loadmacrosfile resetmacros)],
    build => [qw(headernew)],
    rpmdb => [qw(rpmdbverify rpmdbrebuild)],
);

bootstrap Hdlist;

# I18N:
sub N {
    my ($msg, @args) = @_;
    sprintf($msg, @args)
}

# parse* function
# callback => function
#   (
#       header => the header (undef on error)
#       file => actual source
#   )
# files => []
# flags => ??

sub parserpms {
    my (%options) = @_;
    my $db = newdb();
    $db->vsflags($options{checkrpms} ? [ "NOSIGNATURES" ] : [ qw(NOSIGNATURES NOPAYLOAD NODIGESTS) ]);
    foreach my $rpm (@{$options{rpms} || []}) {
        my $header = $db->rpm2header($options{path} ? "$options{path}/$rpm" : $rpm);
        defined($options{callback}) and
            $options{callback}->(
                header => $header,
                dir => $options{path} ? "$options{path}/" : "",
                rpm => $rpm,
            );
    }
}

sub format_rpmpb {
    my (@msgs) = @_;
    my @ret;
    foreach my $p (@msgs) {
        $p->{pb} eq "BADARCH" and do {
            push @ret, N('package %s is intended for a different architecture', $p->{pkg});
            next;
        };
        $p->{pb} eq "BADOS" and do {
            push @ret, N('package %s is intended for a different operating system', $p->{pkg});
            next;
        };
        $p->{pb} eq "PKG_INSTALLED" and do {
            push @ret, N('package %s is allready installed', $p->{pkg});
            next;
        };
        $p->{pb} eq "BADRELOCATE" and do {
            push @ret, N('path %s in package %s is not relocatable', $p->{path}, $p->{pkg});
            next;
        };
        $p->{pb} eq "NEW_FILE_CONFLICT" and do {
            push @ret, N('file %s conflicts between attempted installs of %s and %s', $p->{file}, $p->{pkg}, $p->{pkg2});
            next;
        };
        $p->{pb} eq "FILE_CONFLICT" and do {
            push @ret, N('file %s from install of %s conflicts with file from package %s', $p->{file}, $p->{pkg}, $p->{pkg2});
            next;
        };
        $p->{pb} eq "OLDPACKAGE" and do {
            push @ret, N('package %s (which is newer than %s) is already installed', $p->{pkg2}, $p->{pkg});
            next;
        };
        $p->{pb} eq "DISKSPACE" and do {
            push @ret, N('installing package %s needs %sB on the %s filesystem', $p->{pkg},
                ($p->{size} > 1024 * 1024
                    ? ($p->{size} + 1024 * 1024 - 1) / (1024 * 1024)
                    : ($p->{size} + 1023) / 1024 ) . 
                ($p->{size} > 1024 * 1024 ? 'M' : 'K'),
                $p->{filesystem});
            next;
        };
        $p->{pb} eq "DISKNODES" and do {
            push @ret, N('installing package %s needs %ld inodes on the %s filesystem', $p->{pkg}, $p->{nodes}, $p->{filesystem});
            next;
        };
        $p->{pb} eq "BADPRETRANS" and do {
            push @ret, N('package %s pre-transaction syscall(s): %s failed: %s', $p->{pkg}, $p->{'syscall'}, $p->{error});
            next;
        };
        $p->{pb} eq "REQUIRES" and do {
            push @ret, N('%s is needed by %s%s', $p->{pkg2},
                defined($p->{installed}) ? N("(installed) ") : "",
                $p->{pkg});
            next;
        };
        $p->{pb} eq "CONFLICT" and do {
            push @ret, N('%s conflicts with %s%s', $p->{pkg2},
                defined($p->{val2}) ? N("(installed) ") : "",
                $p->{pkg});
            next;
        };
    };
    @ret
}

1;

__END__

=head1 NAME

Hdlist - perl module to handle hdlist and synthesis files

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 FUNCTIONS

=over 4

=item readconfig($rpmrc, $target)

Force rpmlib to re-read configuration files. If defined, $rpmrc is read.
If $target is defined, rpmlib will read config for this target. $target has
the form "CPU-VENDOR-OS".

    readconfig(); # Reread default configuration
    readconfig(undef, "i386-mandrake-linux"); # Read configuration for i386

=item setverbosity($level)

Set the rpmlib verbosity level. $level can be an integer (0 to 7) or a
verbosity level name.

  - EMERG (0)
  - ALERT (1)
  - CRIT, FATALERROR (2)
  - ERR, ERROR (3)
  - WARNING, QUIET (4)
  - NOTICE, NORMAL (5)
  - INFO, VERBOSE (6)
  - DEBUG (7)

=item setlogcallback(sub {})

Set a perl callback code for rpm logging/output system. When the callback is
set, rpm lets your code print error/information messages. The parameter passed
to the callback is a hash with log value:
    C<locode> => the rpm log code (integer),
    C<priority> => priority of the message (0 to 7),
    C<msg> => the formated string message.
    
Ex:
  setlogcallback( sub {
    my %log = @_;
    print "$log{priority}: $log{msg}\n";
  });

=item lastlogmsg()

Return an array about latest rpm log message information:
  - rpm log code,
  - rpm priority (0 to 7),
  - string message.

=item rpmlog($codelevel, $msg)

Send a message trougth the rpmlib logging system.
  - $codelevel is either an integer value between 0 and 7, or a level code string,
see setverbosity(),
  - $msg is the message to send.

=item format_rpmpb(@pb)

Some functions return an array of rpm transaction problem
(Hdlist::Db->transpb()), this function return an array of human readable
string for each problem.
 
=item querytag()

Returns a hash containing the tags known by rpmlib. The hash has the form
C< TAGNAME => tagvalue >. Note that some tags are virtual and do not have
any tag value, and that some tags are alias to already existing tags, so
they have the same value.

=item tagtypevalue($tagtypename)

Return the type value of a tag type. $tagtypename can be CHAR, INT8, INT16
INT32, STRING, ARRAY_STRING or I18NSTRING. This return value is usefull with
Hdlist::Header::addtag() function.

=item tagName($tagvalue)

Returns the tag name for a given internal value.

    tagName(1000); return "NAME".

See: tagValue().

=item tagValue($tagname)

Returns the internal tag value for C<$tagname>.

    tagValue("NAME"); return 1000.

See: tagName().

=item expand($string)

Evaluate macros contained in C<$string>, like C<rpm --eval>.

    expand("%_var") return "/var".

=item add_macro("_macro value")

Define a macro into rpmlib. The macro is defined for the whole script. Ex:
C<add_macro("_macro value")>. Note that the macro name does have the prefix
"%", to prevent rpm from evaluating it.

=item del_macro("_macro")

Delete a macro from rpmlib. Exactly the reverse of add_macro().

=item loadmacrosfile($filename)

Read a macro configuration file and load macros defined within.
Unfortunately, the function returns nothing, even when file loading failed.

To reset macros loaded from file you have to re-read the rpm config file
with readconfig().

=item resetmacros()

Reset all macros defined with add_macro() functions.

This function does not reset macros loaded with loadmacrosfile().

=item getosname()

Returns the operating system name of current rpm configuration.
Rpmlib auto-detects the system name, but you can force rpm to use
another system name with macros or using readconfig().

=item getarchname()

Returns the arch name of current rpm configuration.
Rpmlib auto-detects the architecture, but you can force rpm to use
another architecture with macros or by using readconfig().

=item buildhost()

Returns the BuildHost name of the current system, ie the value rpm will use
to set BuilHost tag in built rpm.

=item dumprc(*FILE)

Dump rpm configuration into file handle.
Ex:
    dumprc(*STDOUT);

=item dumpmacros(*FILE)

Dump rpm macros into file handle.
Ex:
    dumpmacros(*STDOUT);

=item rpmresign($passphrase, $rpmfile)

Resign a rpm using user settings. C<$passphrase> is the key's gpg/pgp
pass phrase.

Return 0 on success.
    
=item rpmvercmp(version1, version2)

Compare two version and return 1 if left argument is highter, -1 if
rigth argument is highter, 0 if equal.
Ex:
    rpmvercmp("1-1mdk", "2-3mdk"); return -1.

=item headernew()

Returns a new empty Hdlist::Header object.

=item stream2header(*FILE)

Read next header into file handle 'FILE', return a Hdlist::Header object.
If no header data can be found (End of file, invalid data), return undef.

Warning: Perl modifier (like PerlIO::Gzip) won't works.

=item rpm2header(filename)

Extracts rpm header data from rpm filename and return an Hdlist::Header object.
Returns undef if header can't be extracted.

=item installsrpm(filename)

Install a source rpm and return spec file path.
Returns undef if install is impossible.

=item rpmdbinit(rootdir, permissions)

Create an empty rpm database located into I<%{_dbpath}> (useally /var/lib/rpm).
If set, rootdir is the root directory of system where rpm db should be
create, if set, theses permissions will be applied to files, default is 0644.

Directory I<%{_dbpath}> should exist.

Returns 0 on success.

Ex:
    rpmdbinit(); # Create rpm database on the system
    rpmdbinit("/chroot"); # Create rpm database for system located into /chroot.

=item rpmdbverify(rootdir)

Verify rpm database located into I<%{_dbpath}> (useally /var/lib/rpm).
If set, rootdir is root directory of system to check.

Returns 0 on success.

=item rpmdbrebuild(rootdir)

Rebuild the rpm database located into I<%{_dbpath}> (useally /var/lib/rpm).
If set, rootdir is the root directory of system.

Returns 0 on success.

=item specnew(specfile, passphrase, rootdir)

Returns a Hdlist::Spec object after parsing specfile.
If specified, passphare will be used to sign package after build (see
Hdlist::Spec->build()).
If specified, root dir will be use root instead '/'.

Warning: if specfile is not specified, the function returns an empty
Hdlist::Spec object.

=back

=head1 METHODS

=head2 Hdlist::Header

A header is the rpm part that contains all informations. They're
stored as key/value pairs, the key being called 'tag' in rpm-speak.

=over 4

=item Hdlist::Header->write(*FILE)

Dump header data into file handle.

Warning: Perl modifier (like PerlIO::Gzip) won't works.

=item Hdlist::Header->hsize()

Returns the on-disk size of header data, in bytes.

=item Hdlist::Header->copy()

Returns a Hdlist::Header object copy.

=item Hdlist::Header->removetag(tagid)

Remove tag 'tagid' from header.

=item Hdlist::Header->addtag(tagid, tagtype, value1, value2...)

Add a tag into the header:
- tagid is the integervalue of tag
- tagtype is an integer, it identify the tag type to add (see rpmlib headers 
files). Other argument are value to put in tag.

=item Hdlist::Header->listtag()

Returns a list of tag id present in header.

=item Hdlist::Header->hastag(tagid)

Returns true if tag 'tagid' is present in header.

Ex:
    $header->hastag(1000); # Returns true if tag 'NAME' is present.

=item Hdlist::Header->tagtype(tagid)

Returns the tagtype value of tagid. Returns 0 if tagid is not found.

    
=item Hdlist::Header->tag(tagid)

Returns array of tag value for tag 'tagid'.

    $header->tag(1000); # return the name of rpm header.

=item Hdlist::Header->queryformat($query)

Make a formated query on the header, macros in I<$query> are evaluated.
This function works like C<rpm --queryformat ...>

    $header->queryformat("%{NAME}-%{VERSION}-%{RELEASE}");

=item Hdlist::Header->fullname()

In scalar context return the "name-version-version.arch" of package.
In array context return (name, version, release, arch) of package.

=item Hdlist::Header->issrc()

Returns true if package is a source package.

=item Hdlist::Header->compare(header)

Compare the header to another, return 1 if the object is higher, -1 if
header passed as argument is better, 0 if update is not possible.

=back

=head2 Hdlist::Db

=over 4

=item Hdlist::Db->traverse_headers(sub)

Traverse the rpm database and for each header run the callback passed as
argument.

Argument passed to the callback function is the current header as a Hdlist::Header object.

Ex:
    $db->traverse_headers( sub {
        my ($h) = @_;
        print $h->queryformat("%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}");
    });


=item Hdlist::Db->injectheader($header)

Add the header into rpmdb. This is not installing a package, the function
only fill information into the rpmdb.

Return 0 on success.

=item Hdlist::Db->deleteheader($index)

Remove header from rpmdb locate at $index. This is not uninstalling a package,
this function only delete information from rpmdb.

Return 0 on success
    
=item Hdlist::Db->transadd(header, filename, upgrade, relocation, force)

Add rpm headers for next transaction. This means this rpm are going to be
installed on the system.

- header is an Hdlist::Header object,

- filename, if given, is the rpm file you want to install, and should
of course match the header,

- upgrade is a boolean flag to indicate whether the rpm is going to be upgraded
(1 by default).

Returns 0 on success.

See: $Hdlist::Db->transcheck(), $Hdlist::Db->transrun().

=item Hdlist::Db->transremove(rpm_name)

Add rpm to remove for next transaction. This mean the rpm will be uninstalled
from the system.

Argument is the exact rpm name (%{NAME}) or a string as NAME(EPOCH:VERSION-RELEASE).

Returns the number of selected rpms for this transaction.

=item Hdlist::Db->transcheck()

Check current transaction is possible.

Returns 0 on success, 1 if problems are found.

=item Hdlist::Db->transorder()

Call to rpmlib to order the transaction, aka sort rpm installation / 
desintallation.

Returns 0 on success.

=item Hdlist::Db->transrun(callback, flags...)

Really run transaction and install/uninstall packages.

callback can be:

- undef value, let rpm show progression with some default value.

- array ref: each value represent a rpm command line options:

=over 4

    PERCENT: show percentage of progress (--precent)
    HASH: print '#' during progression (--hash)
    LABEL: show rpm name (--verbose)

=back

- code ref: rpm is fully silent, the perl sub is called instead. Arguments
passed to the subroutine are in a hash:

=over 4

    filename => opened filename
    header => current header in transaction
    what => current transaction process
    amount => amount of transaction
    total => number of transaction to do

=back

flags: list of flags to set for transaction (see rpm man page):

=over 4

I<From rpm Transaction flag>:

  - NOSCRIPTS: --noscripts
  - JUSTDB: --justdb
  - NOTRIGGERS: --notriggers
  - NODOCS: --excludedocs
  - ALLFILES: --allfiles
  - DIRSTASH: --dirstash
  - REPACKAGE: --repackage
  - NOTRIGGERPREIN: --notriggerprein
  - NOPRE: --nopre
  - NOPOST: --nopost
  - NOTRIGGERIN: --notriggerin
  - NOTRIGGERUN: --notriggerun
  - NOPREUN: --nopreun
  - NOPOSTUN: --nopostun
  - NOTRIGGERPOSTUN: --notriggerpostun
  - NOSUGGEST: --nosuggest
  - NOMD5: --nomd5
  - ADDINDEPS: --aid
  - noscripts: Do not running any scripts, neither triggers

I<From rpm prob filter>

  - IGNOREOS: --ignoreos
  - IGNOREARCH: --ignorearch
  - REPLACEPKG: --replacepkgs
  - REPLACENEWFILES: --replacefiles
  - REPLACEOLDFILES: --replacefiles
  - OLDPACKAGES: --oldpackage
  - DISKSPACE: --ignoresize
  - DISKNODE: --ignoresize
  - ignoresize: set DISKNODE and DISKSPACE
  - replacefiles: set both REPLACENEWFILES and REPLACEOLDFILES

=back

Returns 0 on success.

=item Hdlist::Db->transpb()

To be run after Hdlist::Db->transcheck() or Hdlist::Db->transrun(). Returns an
array of problems found during transaction. Each value in the array is a hash
where $pbs_found[n]->{pb} is the problem type: BADARCH, BADOS, PKG_INSTALLED,
BADRELOCATE, NEW_FILE_CONFLICT, FILE_CONFLICT, OLDPACKAGE, DISKSPACE,
DISKNODES, BADPRETRANS, REQUIRES, CONFLICT, UNKNOWN.

For each problem type, values in hash are:

  - BADARCH, BADOS, PKG_INSTALLED:
    $pbs_found[n]->{pkg}: the package name causing the problem
  - BADRELOCATE
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{path}: not reloacatable path
  - NEW_FILE_CONFLICT, FILE_CONFLICT
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{pkg2}: the second package in conflict
  - OLDPACKAGE
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{pkg2}: the second package
  - DISKSPACE
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{size}: the missing space in bytes
    $pbs_found[n]->{filesystem}: the filesystem where space is missing
  - DISKNODES
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{nodes}: the missing nodes number
    $pbs_found[n]->{filesystem}: the filesystem where nodes are missing
  - BADPRETRANS
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{syscall}: syscall name
    $pbs_found[n]->{error}: the error number
  - REQUIRES, CONFLICT
    $pbs_found[n]->{pkg}: the package name causing the problem
    $pbs_found[n]->{pkg2}: the requires/conflict name
    $pbs_found[n]->{installed}: is the pkg2 installed
  - UNKNOWN
    nothing is set.

=head2 Hdlist::Spec

=over 4

=item Hdlist::Spec->srcheader()

Returns a Hdlist::Header object like source rpm will be.
Please notice that the header is not finish and header information you'll
get can be incomplete, it depend if you call the function before or after
Hdlist::Spec->build().

=item Hdlist::Spec->srcrpm()

Returns the source filename spec file will build. The function take care
about rpmlib configuration (build dir path).

=item Hdlist::Spec->binrpm()

Returns files names of binaries rpms that spec will build. The function take
care about rpmlib configuration (build dir path).

=item Hdlist::Spec->build([ @actions ])

Run build process on spec file.
Each value in @actions is one or more actions to do:

  - PACKAGESOURCE: build source package,
  - PREP: run prep stage,
  - BUILD: run build stage,
  - INSTALL: run install stage,
  - CHECK: check installed files,
  - FILECHECK: check installed files,
  - PACKAGEBINARY: build binaries packages,
  - CLEAN: run clean stage,
  - RMSPEC: delete spec file,
  - RMSOURCE: delete sources files,
  - RMBUILD: delete build files,

  - bp: alias for PREP
  - bc: alias for PREP, BUILD
  - bi: alias for PREP, BUILD, INSTALL, CHECK
  - bl: alias for PREP, BUILD, INSTALL, CHECK, FILECHECK
  - bb: alias for PREP, BUILD, INSTALL, CHECK, FILECHECK, PACKAGEBINARY, CLEAN
        (full binaries packages build)
  - bs: alias for PACKAGESOURCE
  - ba: alias for bb and bs

=back

=head1 SEE ALSO

L<rpm(8)>,

This aims at replacing part of the functionality provided by URPM.

=cut
