##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Buildmedia.pm,v 1.15 2005/01/11 20:17:52 othauvin Exp $

package Hdlist::Buildmedia;

use strict;
use warnings;
use Hdlist;
use Hdlist::Index;
use Hdlist::Sign;
use File::Find;
use Digest::MD5;
use Packdrakeng;

sub new {
    my ($class, %options) = @_;

    my $Buildmedia = {
        hdlist => 'hdlist.cz',
        synthesis => 'synthesis.hdlist.cz',
        md5sum => 'MD5SUM',
        list => 'list',
        version => 'VERSION',

        media_version => undef,
        media_name => undef,

        topdir => undef, # rpms dir

        infosdir => undef, # index dir
        recursive => 1,

        sig => 0,
        _signature => undef,
        sigpath => undef,
        signame => undef,
        passphrase => undef,
        password_file => undef,
 
        log => sub {
            my ($m, @v) = @_;
            printf STDERR "$m\n", @v;
        },
    };
    
    foreach (keys(%$Buildmedia)) {
        $Buildmedia->{$_} = $options{$_} if (exists($options{$_}));
    }
    $Buildmedia->{infosdir} ||= $Buildmedia->{topdir};

    if ($Buildmedia->{version} && open(my $h_version, "<", "$Buildmedia->{infosdir}/$Buildmedia->{version}")) {
        my @l = <$h_version> =~ m/(.*) for (.*)/;
        $Buildmedia->{media_name} ||= $l[0];
        $Buildmedia->{media_version} ||= $l[1];
        close($h_version);
    }
    
    -d $Buildmedia->{topdir} || -l $Buildmedia->{topdir} or return undef;
    
    bless($Buildmedia, $class);
}

sub clean {
    my ($self) = @_;
    foreach (qw(hdlist synthesis list md5sum version)) {
        $self->{log}->("Unlink %s", "$self->{infosdir}/$self->{$_}");
        unlink("$self->{infosdir}/$self->{$_}");
    }
}

sub medianame {
    my ($self) = @_;
    return $self->{media_name} ? "$self->{media_name} [$self->{topdir}]" : $self->{topdir};
}

sub findrpms {
    my ($self, %options) = @_;
    my @files;
    $self->{rpms} = [];
    $self->{requiredfiles} = [];
    my $result = 1;
    $self->{log}->("Findind rpms%s in [%s] (rpm md5: %s)",
        $self->{recursive} ? " (recursively)" : "",
        $self->{topdir},
        $options{checkrpms} ? "on" : "off");
        
    my $db = newdb();
    $db->vsflags($options{checkrpms} ? [ "NOSIGNATURES" ] : [ qw(NOSIGNATURES NOPAYLOAD NODIGESTS) ]);

    my $sign =undef;
    if ($self->{sig}) {
        $sign = Hdlist::Sign->new(
            _signature => $self->{sigtype},
            name => $self->{signame},
            path => $self->{sigpath},
            password_file => $self->{password_file},
            passphrase => $self->{passphrase},
            checkrpms => 0,

            log => $self->{log},
        );
    }
    
    find( {

            wanted => sub {
                -d $_ and do {
                    !$self->{recursive} && $_ ne '.' and do {
                        $File::Find::prune = 1;
                        return;
                    };
                };

                m/\.rpm$/ or return;

                my $header = $db->rpm2header($_);

                if(defined($options{callback})) {
                    $options{callback}->(
                        header => $header,
                        rpm => $_,
                    ) or return;
                }
                my ($relativepath) = $File::Find::name =~ m:^$self->{topdir}/*(.*):;
                $header or do {
                    $self->{log}->("Error: Can't read %s", $relativepath);
                    $options{nobadrpm} and $result = 0;
                    return;
                };

                if ($self->{sig}) {
                    $sign->rpmsign($File::Find::name, $header) < 0 and do {
                        $self->{log}->("Error: Can't resign %s", $File::Find::name);
                    };
                }
                push(@{$self->{requiredfiles}}, $header->requiredfiles());
                my %uniq; @uniq{@{$self->{requiredfiles}}} = (); @{$self->{requiredfiles}} = keys %uniq;
                push(@{$self->{rpms}}, $relativepath);
            },
            follow => 1,
            follow_skip => 2,
        },
        $self->{topdir}
    );
    
    @{$self->{rpms}} = sort(@{$self->{rpms}});
    $result;
}

sub needbuild {
    my ($self) = @_;
    my $needrebuild = 0;

    foreach (qw(hdlist synthesis version md5sum)) {
        defined($self->{$_}) && ! -e "$self->{infosdir}/$self->{$_}" and return 1;
    }

    # checking if rebuild is need

    my $hdlistdate = (stat("$self->{infosdir}/$self->{hdlist}"))[7];

    foreach (@{$self->{rpms} || []}) {
        (stat("$self->{topdir}/$_"))[7] >= $hdlistdate and return 1;
    }

    my %indexedrpms;
    if ($self->{hdlist} && (my $pack = Packdrakeng->open(archive => "$self->{infosdir}/$self->{hdlist}"))) {
        my (undef, $f, undef) = $pack->getcontent();
        @indexedrpms{ map { $_ .= ".rpm"; $_ } @$f} = ();
        $pack = undef;
    }

    foreach my $rpm (@{$self->{rpms}}) {
        my $file = substr($rpm, rindex($rpm, '/')+1);
        if(exists $indexedrpms{$file}) {
            delete($indexedrpms{$file});
        } else {
            $needrebuild = 1;
            last;
        }   
    }
    keys %indexedrpms and $needrebuild = 1;
    
    $needrebuild;
}

sub foundrpms {
    my ($self) = @_;
    @{$self->{rpms} || []};
}

sub requiredfiles {
    my ($self) = @_;
    @{$self->{requiredfiles} || []}
}

sub build {
    my ($self, %options) = @_;
    
    my $result = 0;
    
    @{$self->{rpms} || []} or $self->findrpms(
        checkrpms => $options{checkrpms},
        nobadrpm => $options{nobadrpm},
    ) or return;
    $options{filestoprovides} ||= $self->{requiredfiles};
    
    $options{clean} and $self->clean();
   
    if ($options{nobuild}) { return 0 };
    
    $self->{log}->("Building in %s: %s, %s, %s, %s, %s",
        $self->{infosdir},
        $self->{hdlist} || '(no hdlist)',
        $self->{synthesis} || '(no synthesis)',
        $self->{list} || '(no list)',
        $self->{version} || '(no version)',
        $self->{md5sum} || '(no md5)',
    );

    if ($options{force} || $self->needbuild()) {
        $result = 1;
        Hdlist::Index::buildindex(
            path => $self->{topdir},
            hdlist => $self->{hdlist} ? "$self->{infosdir}/$self->{hdlist}" : undef,
            synthesis => $self->{synthesis} ? "$self->{infosdir}/$self->{synthesis}" : undef,
            list => $self->{list} ? "$self->{infosdir}/$self->{list}" : undef,
            rpms => $self->{rpms},
            filestoprovides => $options{filestoprovides},
            complevel => 9,
        ) or do {
            $self->{log}->("Error: Unable to create index files");
            $result = -1;
        };

        if ($self->{md5sum}) {
            if (open(my $h_md5, ">", "$self->{infosdir}/$self->{md5sum}") or die "Can't open $self->{infosdir}/$self->{md5sum}") {
                $self->genmd5($h_md5);
                close($h_md5);
            } else {
                $self->{log}->("Error: Unable to create md5sum files");
                $result = -1;
            }
        }

        if ($self->{version}) {
            if (open(my $h_version, ">", "$self->{infosdir}/$self->{version}")) {
                my @d = gmtime(time);
                printf $h_version "%s for %s\n%04d%02d%02d %02d:%02d\n",
                    $self->{media_name} || '(no name)',
                    $self->{media_version} || '(unknown version)',
                    $d[5] + 1900, $d[4] + 1, $d[3],
                    $d[2], $d[1];
                close($h_version);
            } else {
                $self->{log}->("Error: Can't write version file: %s", "$self->{infosdir}/$self->{version}");
                $result = -1;
            }
        }
            
    } else {
        $self->{log}->("Rebuilding is not needed");
    }
    return $result;
}

sub genmd5 {
    my ($self, $handle) = @_;
    foreach my $file (qw(hdlist synthesis)) {
        my $md5 = Digest::MD5->new;
        $self->{$file} or next;
        open(my $hf, "<", "$self->{infosdir}/" . $self->{$file}) or do {
            $self->{log}->("Error: Unable to open %s during md5sum", "$self->{infosdir}/" . $self->{$file});
            next;
        };
        $md5->addfile($hf);
        close($hf);
        printf($handle "%s  %s\n", $md5->hexdigest, $self->{$file});
    }
}

1;

__END__

=head1 NAME

Hdlist::Buildmedia - Perl module to generate mandrake repository

=head1 SYNOPSIS

use Hdlist::Buildmedia;

my $media = Hdlist::Buildmedia->new(
    rpmsdir => "/my/rpms/dir",
    recursive => 1,
);
$media->build();

=head1 DESCRIPTION

This module contains basics fonctions to generate hdlist (header collection,
uses by urpmi), synthesis (uses by urpmi) and headers files (uses by yum).

=head1 FUNCTIONS

=item B<Hdlist::Buildmedia->new(%arg)>

Return a new Buildmedia object.

%arg:

- topdir: the toplevel dir containing rpms
- infosdir: where index files will be generated (default: topdir)
- recursive: looking for rpms recursivelly (default: false)
- hdlist: the name of hdlist file (default: hdlist.cz)
- synthesis: the name of the synthesis file (default: synthesis.hdlist.cz)
- list: the name of the list file (default: list)
- md5sum: the name of the md5sum file (default: MD5SUM)
- version: the name of the version file (default: VERSION)
- media_name: the name of the media
- media_version: the version of media

=item B<$media->clean()>

Delete index files

=item B<$media->findrpms(%options)>

Find rpms into rpmsdir.

options:
- checkrpms: if set, integrity of rpms will be checked.
- callback: if set, the function will be called, if the return value if false
the rpm will not be selected.
- nobadrpm: return 0 if a bad rpm is found

Return a list of of file find as requirement. This list should be given to
build() function to properly populate the synthesis file.

=item B<$media->medianame()>

Return the name and path of media

=item B<$media->foundrpms()>

Return an array of rpms files found and selected after $media->findrpms();

=item B<$media->requiredfiles()>

Return an array about files found as requirement

=item B<$media->(needbuild)>

Return true if index files need to be rebuild by inspecting hdlist.

=item B<$media->build(%options)>

Generate index files.

options:
- force: if set, will build index files even a rebuild is not need
- clean: if set, index files will deleted before build
- nobadrpm: stop build if bad rpm is found

=item B<$media->genmd5($handle)>

Print md5sum of hdlist/synthesis into $handle.

=head1 SEE ALSO

L<rpm(8)>, L<Packdrakeng>

This aims at replacing part of the functionality provided by URPM.

=cut
